import { firstCharToLowerCase, Subject } from '@/utils/tool';
import { useEffect, useState } from 'react';
import { DATA } from './API';

export const isClient = !!window.ipcRenderer; // process.env.IS_CLIENT;

class FileMessageManager extends Subject {
  public list: DATA.SocketFileMsg[] = [];
  // 主键 taskId
  public fileMap = new Map();

  addFileMsg(message: DATA.SocketFileMsg) {
    this.list = [message].concat(this.list);
    this.notifyObservers(this.list);
  }
  updateFileMsg(message: DATA.SocketFileMsg) {
    this.fileMap.set(message.taskId, message);
    this.list = this.list.map((msg) =>
      msg.taskId === message.taskId ? message : msg,
    );
    this.notifyObservers(this.list);
  }
  receiveMessage(messageStr: string) {
    try {
      const message = firstCharToLowerCase(
        JSON.parse(messageStr),
      ) as DATA.SocketFileMsg;
      const existMsg = this.fileMap.get(message.taskId);
      if (existMsg) {
        this.updateFileMsg(message);
      } else {
        this.addFileMsg(message);
      }
    } catch (e) {
      console.error('message 解析失败:', e, messageStr);
    }
  }
}

const fileMng = new FileMessageManager();

if (isClient) {
  window.addIpcRendererListener('socket:on-message', (_, message) => {
    console.log('receive electron socket message:', message);
    fileMng.receiveMessage(message);
  });
  window.addIpcRendererListener('socket:on-error', (_, ...args) => {
    console.log('receive electron socket error:', args);
  });
  window.addIpcRendererListener('socket:on-close', (_, ...args) => {
    console.log('receive electron socket close:', args);
  });
}

export function useSocketMessages() {
  const [list, setList] = useState(fileMng.list);
  useEffect(() => {
    fileMng.addObserver(setList);
    return () => {
      fileMng.removeObserver(setList);
    };
  }, []);

  return list;
}
